package main

import (
	"context"
	"fmt"

	"dehub.dev/src/dehub.git"
	"dehub.dev/src/dehub.git/cmd/dehub/dcmd"

	"gopkg.in/src-d/go-git.v4/plumbing"
)

func cmdVerify(ctx context.Context, cmd *dcmd.Cmd) {
	flag := cmd.FlagSet()
	rev := flag.String("rev", "HEAD", "Revision of commit to verify")
	branch := flag.String("branch", "", "Branch that the revision is on. If not given then the currently checked out branch is assumed")

	var repo repo
	repo.initFlags(flag)

	cmd.Run(func() (context.Context, error) {
		if err := repo.openRepo(); err != nil {
			return nil, err
		}

		gitCommit, err := repo.GetGitRevision(plumbing.Revision(*rev))
		if err != nil {
			return nil, fmt.Errorf("resolving revision %q: %w", *rev, err)
		}
		gitCommitHash := gitCommit.GitCommit.Hash

		var branchName plumbing.ReferenceName
		if *branch == "" {
			if branchName, err = repo.ReferenceToBranchName(plumbing.HEAD); err != nil {
				return nil, fmt.Errorf("determining branch at HEAD: %w", err)
			}
		} else {
			branchName = plumbing.NewBranchReferenceName(*branch)
		}

		if err := repo.VerifyCommits(branchName, []dehub.GitCommit{gitCommit}); err != nil {
			return nil, fmt.Errorf("could not verify commit at %q (%s): %w",
				*rev, gitCommitHash, err)
		}

		fmt.Printf("commit at %q (%s) is good to go!\n", *rev, gitCommitHash)
		return nil, nil
	})
}
